from urlparse import urljoin
from django import forms
from django.utils.translation import ugettext_lazy as _
from i18nurl import settings


class LanguageSelectionForm(forms.Form):
    """Language selection form."""
    language = forms.ChoiceField(
        label=_('language'),
        choices=settings.I18N_LANGUAGES,
    )
    next = forms.CharField(
        label=_('next'),
        required=False,
        widget=forms.HiddenInput,
    )
    temporary = forms.BooleanField(
        label=_('remember'),
        required=False,
        initial=False,
        widget=forms.HiddenInput,
    )



# -*- coding: utf-8 -*-
from django.core.urlresolvers import reverse
from django.utils import translation


def reverse_i18n(url, language, *args, **kwargs):
    """Return the i18n url in a specific language."""
    cur_language = translation.get_language()
    try:
        translation.activate(language)
        try:
            url = reverse(url, *args, **kwargs)
        except:
            url = reverse(url)
    finally:
        translation.activate(cur_language)
    return url



"""Context processors."""


def i18n(request):
    """Context processor which returns a dictionary containing I18N_SITES."""
    from i18nurl import settings
    return {'I18N_LANGUAGES': settings.I18N_LANGUAGES,
            'I18N_REDIRECT_URL_NAME': settings.I18N_REDIRECT_URL_NAME}



# -*- coding: utf-8 -*-
from django.conf import settings
from django.template.base import Library, TemplateSyntaxError, kwarg_re, Node
from django.utils.encoding import smart_text
from django.core.urlresolvers import NoReverseMatch

from .. import reverse_i18n

register = Library()


class URLNode(Node):
    def __init__(self, view_name, language, args, kwargs, asvar):
        self.view_name = view_name
        self.language = language
        self.args = args
        self.kwargs = kwargs
        self.asvar = asvar

    def render(self, context):
        args = [arg.resolve(context) for arg in self.args]
        kwargs = dict([(smart_text(k, 'ascii'), v.resolve(context))
                       for k, v in self.kwargs.items()])

        view_name = self.view_name.resolve(context)
        language = self.language.resolve(context)

        if not view_name:
            raise NoReverseMatch(
                "'url' requires a non-empty first argument. "
                "The syntax changed in Django 1.5, see the docs.")

        # Try to look up the URL twice: once given the view name, and again
        # relative to what we guess is the "main" app. If they both fail,
        # re-raise the NoReverseMatch unless we're using the
        # {% url ... as var %} construct in which case return nothing.
        url = ''
        try:
            url = reverse_i18n(view_name, language,
                               args=args, kwargs=kwargs,
                               current_app=context.current_app)
        except NoReverseMatch as e:
            if settings.SETTINGS_MODULE:
                project_name = settings.SETTINGS_MODULE.split('.')[0]
                try:
                    url = reverse_i18n(project_name + '.' + view_name,
                                       language,
                                       args=args, kwargs=kwargs,
                                       current_app=context.current_app)
                except NoReverseMatch:
                    if self.asvar is None:
                        # Re-raise the original exception, not the one with
                        # the path relative to the project. This makes a
                        # better error message.
                        raise e
            else:
                if self.asvar is None:
                    raise e

        if self.asvar:
            context[self.asvar] = url
            return ''
        else:
            return url


@register.tag
def i18nurl(parser, token):
    """
    Returns an absolute URL matching given view with its parameters in
    the good language.

        {% i18nurl "path.to.some_view" "language" arg1 arg2 %}

        or

        {% i18nurl "path.to.some_view" "language" name1=value1 name2=value2 %}

    """
    bits = token.split_contents()
    if len(bits) < 3:
        raise TemplateSyntaxError("'%s' takes at least two argument"
                                  " (path to a view and language)" % bits[0])
    try:
        viewname = parser.compile_filter(bits[1])
    except TemplateSyntaxError as exc:
        exc.args = (exc.args[0] + ". The syntax of 'url' "
                    "changed in Django 1.5, see the docs."),
        raise

    language = parser.compile_filter(bits[2])

    args = []
    kwargs = {}
    asvar = None
    bits = bits[3:]
    if len(bits) >= 2 and bits[-2] == 'as':
        asvar = bits[-1]
        bits = bits[:-2]

    if len(bits):
        for bit in bits:
            match = kwarg_re.match(bit)
            if not match:
                raise TemplateSyntaxError("Malformed arguments to url tag")
            name, value = match.groups()
            if name:
                kwargs[name] = parser.compile_filter(value)
            else:
                args.append(parser.compile_filter(value))

    return URLNode(viewname, language, args, kwargs, asvar)


class CurrentURLNode(Node):
    def __init__(self, language, asvar):
        self.language = language
        self.asvar = asvar

    def render(self, context):
        try:
            resolver_match = context['request'].resolver_match
        except AttributeError:
            return ''
        app_name = resolver_match.app_name
        url_name = resolver_match.url_name
        args = resolver_match.args
        kwargs = resolver_match.kwargs

        language = self.language.resolve(context)

        if app_name:
            view_name = '{app_name}:{url_name}'.format(app_name=app_name,
                                                       url_name=url_name)
        else:
            view_name = '{url_name}'.format(url_name=url_name)

        # Try to look up the URL twice: once given the view name, and again
        # relative to what we guess is the "main" app. If they both fail,
        # re-raise the NoReverseMatch unless we're using the
        # {% url ... as var %} construct in which case return nothing.
        url = ''
        try:
            url = reverse_i18n(view_name, language,
                               args=args, kwargs=kwargs,
                               current_app=context.current_app)
        except NoReverseMatch as e:
            if settings.SETTINGS_MODULE:
                project_name = settings.SETTINGS_MODULE.split('.')[0]
                try:
                    url = reverse_i18n(project_name + '.' + view_name,
                                       language,
                                       args=args, kwargs=kwargs,
                                       current_app=context.current_app)
                except NoReverseMatch:
                    if self.asvar is None:
                        # Re-raise the original exception, not the one with
                        # the path relative to the project. This makes a
                        # better error message.
                        raise e
            else:
                if self.asvar is None:
                    raise e

        if self.asvar:
            context[self.asvar] = url
            return ''
        else:
            return url


@register.tag
def current_i18nurl(parser, token):
    """
    Returns the current page absolute url in the right language.

        {% current_i18nurl "language" %}

    """

    bits = token.split_contents()
    asvar = None

    if len(bits) < 2:
        raise TemplateSyntaxError("'%s' takes one argument"
                                  " (language)" % bits[0])
    language = parser.compile_filter(bits[1])

    bits = bits[3:]
    if len(bits) >= 2 and bits[-2] == 'as':
        asvar = bits[-1]
        bits = bits[:-2]

    return CurrentURLNode(language, asvar)



from django.conf.urls import patterns, url

from i18nurl.views import guess_language, set_language


urlpatterns = patterns(
    '',
    url(r'^$', guess_language, name='guess_language'),
    url(r'^i18n/$', set_language, name='set_language'),
)



"""Views to manage active language."""
from django.conf import settings
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.utils.translation import override
from django.views.generic import FormView, RedirectView

from i18nurl.settings import I18N_REDIRECT_URL_NAME
from i18nurl.forms import LanguageSelectionForm


class GuessLanguageView(RedirectView):
    """Guess language and redirect to the matching URL."""
    permanent = False

    def get_redirect_url(self, **kwargs):
        """Try to guess language from the request and to redirect to
        the matching language.

        If not able to guess the adequate site, redirects to the
        default site and display a message which includes language
        selection links.

        """
        return reverse(I18N_REDIRECT_URL_NAME)


guess_language = GuessLanguageView.as_view()


class SetLanguageView(FormView):
    """Display language selection form, or, if form submitted, remember
    selected language and redirects to the adequate URL."""
    form_class = LanguageSelectionForm
    template_name = 'i18n/set_language.html'

    def form_valid(self, form):
        """Rememeber selected language and redirects to the adequate URL."""
        #next = self.request.META.get('HTTP_REFERER', next)
        language_code = form.cleaned_data['language']
        redirect_url = form.cleaned_data['next']
        remember = not form.cleaned_data['temporary']
        if not redirect_url:
            with override(language_code):
                redirect_url = reverse(I18N_REDIRECT_URL_NAME)
        response = HttpResponseRedirect(redirect_url)
        if remember:
            # Write in user's profile.
            user = self.request.user
            if hasattr(user, 'language_code') and \
                    user.language_code != language_code:
                user.language_code = language_code
                user.save()
            # Write in session.
            if hasattr(self.request, 'session'):
                self.request.session['django_language'] = language_code
            # Write in a cookie.
            else:
                response.set_cookie(settings.LANGUAGE_COOKIE_NAME,
                                    language_code)
        return response


set_language = SetLanguageView.as_view()



import locale
import re

from django.conf import settings
from django.utils import translation
from django.utils.translation.trans_real import (check_for_language,
                                                 parse_accept_lang_header,
                                                 to_locale)
from django.utils.cache import patch_vary_headers

from .utils import is_language_supported


class BaseLanguageMiddleware(object):
    def get_language_from_request(self, request):
        raise NotImplementedError()

    def process_request(self, request):
        language_code = getattr(request, 'LANGUAGE_CODE', None)
        if language_code is None:
            language_code = self.get_language_from_request(request)
            if language_code:
                translation.activate(language_code)
                request.LANGUAGE_CODE = language_code
        return None

    def process_response(self, request, response):
        patch_vary_headers(response, ('Accept-Language',))
        if 'Content-Language' not in response:
            response['Content-Language'] = translation.get_language()
        translation.deactivate()
        return response


class DefaultLanguageMiddleware(BaseLanguageMiddleware):
    """Middleware that activate settings.LANGUAGE_CODE."""
    def get_language_from_request(self, request):
        """Return settings.LANGUAGE_CODE."""
        return settings.LANGUAGE_CODE


class UrlPrefixLanguageMiddleware(BaseLanguageMiddleware):
    """Looks after a language prefix in request.path_info."""
    def get_language_from_request(self, request):
        """Search one of settings.LANGUAGES code in request.path_info."""
        alternatives = [code for code, name in settings.LANGUAGES]
        pattern = r'^/(?P<language>%s)(?P<path>/.*|)$' \
                  % '|'.join(alternatives)
        match = re.match(pattern, request.path_info)
        if match is not None:
            requested_language = match.group('language')
            return is_language_supported(requested_language)
        return None


class CookieLanguageMiddleware(BaseLanguageMiddleware):
    def get_language_from_request(self, request):
        lang_code = request.COOKIES.get(settings.LANGUAGE_COOKIE_NAME)
        return is_language_supported(lang_code)


class SessionLanguageMiddleware(BaseLanguageMiddleware):
    def get_language_from_request(self, request):
        if hasattr(request, 'session'):
            lang_code = request.session.get('django_language', None)
            return is_language_supported(lang_code)
        return None


class HttpAcceptLanguageMiddleware(BaseLanguageMiddleware):
    def get_language_from_request(self, request):
        supported = dict(settings.LANGUAGES)
        accept = request.META.get('HTTP_ACCEPT_LANGUAGE', '')
        for accept_lang, unused in parse_accept_lang_header(accept):
            if accept_lang == '*':
                break

            # We have a very restricted form for our language files
            # (no encoding specifier, since they all must be UTF-8 and
            # only one possible language each time. So we avoid the
            # overhead of gettext.find() and work out the MO file
            # manually.

            # 'normalized' is the root name of the locale in POSIX
            # format (which is the format used for the directories
            # holding the MO files).
            normalized = locale.locale_alias.get(to_locale(accept_lang, True))
            if not normalized:
                continue
            # Remove the default encoding from locale_alias.
            normalized = normalized.split('.')[0]

            for lang_code in (accept_lang, accept_lang.split('-')[0]):
                lang_code = lang_code.lower()
                if lang_code in supported and check_for_language(lang_code):
                    return lang_code
        return None


class UserLanguageMiddleware(BaseLanguageMiddleware):
    def get_language_from_request(self, request):
        user = request.user
        requested_language = None
        if hasattr(user, 'language_code'):
            requested_language = user.language_code
        language = is_language_supported(requested_language)
        if language:
            return language
        return None



from django.conf import settings

I18N_LANGUAGES = getattr(settings, 'I18N_LANGUAGES',
                         settings.LANGUAGES)

I18N_REDIRECT_URL_NAME = getattr(settings, 'I18N_REDIRECT_URL_NAME',
                                 'home')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from os.path import abspath, dirname, join
from setuptools import find_packages, setup


def read_relative_file(filename):
    """Returns contents of the given file, whose path is supposed relative
    to this module."""
    with open(join(dirname(abspath(__file__)), filename)) as f:
        return f.read()


if __name__ == '__main__':  # ``import setup`` doesn't trigger setup().
    setup(name='django-i18nurl',
          version=read_relative_file('VERSION').strip(),
          description="Django i18n URL",
          long_description=read_relative_file('README.rst'),
          classifiers=['Development Status :: 3 - Alpha',
                       'Environment :: Web Environment',
                       'Framework :: Django',
                       'Intended Audience :: Developers',
                       'License :: OSI Approved :: BSD License',
                       'Natural Language :: English',
                       'Programming Language :: Python',
                       'Programming Language :: Python :: 3',
          ],
          keywords='django i18n url',
          author='Rémy Hubscher',
          author_email='hubscher.remy@gmail.com',
          url='https://github.com/novagile/django-i18nurl',
          license='BSD Licence',
          packages=find_packages(),
          include_package_data=True,
          zip_safe=False,
          install_requires=[]  # depends on Django
    )



import locale

from django.utils.translation.trans_real import to_locale


def normalize_language(language):
    return locale.locale_alias.get(to_locale(language, True))


def is_language_supported(language, supported_languages=None):
    if supported_languages is None:
        from django.conf import settings
        supported_languages = dict(settings.LANGUAGES).keys()
    if not language:
        return None
    normalized = normalize_language(language)
    if not normalized:
        return None
    # Remove the default encoding from locale_alias.
    normalized = normalized.split('.')[0]
    for lang in (normalized, normalized.split('_')[0]):
        if lang.lower() in supported_languages:
            return lang
    return None
